;----------------------------------------------------------------------------
; macros.inc --
;----------------------------------------------------------------------------

; Block size definition
; Can adjust to conserve ROM space
;
.define                 BLKSIZE         $10
.define                 BLKMASK         (BLKSIZE-1)

.define                 NOPSIZE         $10
.define                 HALTSIZE        $10
.define                 OUT0SIZE        $10

; Repeatedly define an opcode
;
                        .macro  defop
                        .rept \1
                        \2
                        .endr
                        .endm

; NOP delay
;
                        .macro delay
                        defop \1, nop
                        .endm

; Define a SMS CRAM entry
;
                        .macro  defcol
                        .db     ((\1&3)<<0)|((\2&3)<<2)|((\3&3)<<4)
                        .endm

; Output data to port (in blocks)
; Trashes c,hl
;
.macro                  outix
                        .define OUTIX_BLKCNT\@ (\1/BLKSIZE)
                        .define OUTIX_BLKREM\@ (\1&BLKMASK)
                        .rept   OUTIX_BLKCNT\@
                        call    outiblk-(BLKSIZE*2)
                        .endr
                        .ifneq  OUTIX_BLKREM\@ 0
                        call    outiblk-(OUTIX_BLKREM\@*2)
                        .endif
                        .endm

; Input data from port (in blocks)
; Trashes c,hl
;
.macro                  inix
                        .define INIX_BLKCNT\@ (\1/BLKSIZE)
                        .define INIX_BLKREM\@ (\1&BLKMASK)
                        .rept   INIX_BLKCNT\@
                        call    iniblk-(BLKSIZE*2)
                        .endr
                        .ifneq  INIX_BLKREM\@ 0
                        call    iniblk-(INIX_BLKREM\@*2)
                        .endif
                        .endm

; Copy data (in blocks)
; Trashes hl,de,bc
;
.macro                  ldix  
                        .define LDIX_BLKCNT\@ (\1/BLKSIZE)
                        .define LDIX_BLKREM\@ (\1&BLKMASK)
                        .rept   LDIX_BLKCNT\@
                        call    ldiblk-(BLKSIZE*2)
                        .endr
                        .ifneq  LDIX_BLKREM\@ 0
                        call    ldiblk-(LDIX_BLKREM\@*2)
                        .endif
                        .endm

; Upload memory to CRAM
; Trashes a,bc,hl
;
.macro                  loadcram
                        ld      c, $BF
                        ld      a, \1&$FF
                        out     (c), a
                        ld      a, $C0|((\1>>8)&$3F)
                        out     (c), a
                        dec     c
                        ld      hl, \2
                        outix   \3
                        .endm

; Upload memory to VRAM
; Trashes a,bc,hl
;
.macro                  loadvram
                        ld      c, $BF
                        ld      a, \1&$FF
                        out     (c), a
                        ld      a, $40|((\1>>8)&$3F)
                        out     (c), a
                        dec     c
                        ld      hl, \2
                        outix   \3
                        .endm

; Download VRAM data to memory
; Trashes a,bc,hl
;
.macro                  getvram
                        ld      c, $BF
                        ld      a, \1&$FF
                        out     (c), a
                        ld      a, (\1>>8)&$3F
                        out     (c), a
                        dec     c
                        ld      hl, \2
                        inix   \3
                        .endm

; memcpy (srcaddr, dstaddr, size)
; Trashes bc,de,hl
;
.macro                  memcpy
                        ld      hl, \1
                        ld      de, \2
                        ldix    \3
                        .endm

; memset (srcaddr, fillvalue, size)
; Trashes bc,de,hl
;
.macro                  memset
                        ld      hl, \1
                        ld      de, \1+1
                        ld      (hl), \2
                        ldix    \3
                        .endm

; Set name table address (M4)
.macro                  setnt
                        ld      hl, $82F1|((\1>>10)&$0E)
                        rst     10h
                        .endm

; Set sprite table address (M4)
.macro                  setsat
                        ld      hl, $8581|((\1>>7)&$7E)
                        rst     10h
                        .endm

; Set PN address (TMS)
.macro                  setpn
                        ld      hl, $82F0|((\1>>10)&$0F)
                        rst     10h
                        .endm

; Set CT address (TMS)
.macro                  setct
                        ld      hl, $8300|((\1>>6)&$FF)
                        rst     10h
                        .endm

; Set PG address (TMS)
.macro                  setpg
                        ld      hl, $84F8|((\1>>11)&$07)
                        rst     10h
                        .endm

; Set SA address (TMS)
.macro                  setsa
                        ld      hl, $8580|((\1>>7)&$7F)
                        rst     10h
                        .endm

; Set SG address (TMS)
.macro                  setsg
                        ld      hl, $86F8|((\1>>11)&$07)
                        rst     10h
                        .endm

;----------------------------------------------------------------------------
; End
;----------------------------------------------------------------------------
