/*
    main.c --
    Creates crossfade tables.
*/

#include <stdio.h>
#include <math.h>

#define DEBUG   0

typedef unsigned char uint8;

uint8 bg_cram[] = {01, 0x02, 0x10, 0x20};   /* Background colors */
uint8 fg_cram = 0x3F;                       /* Foreground (image) color */
uint8 ramp[4][16];

uint8 interpolate(uint8 src, uint8 dst, int step)
{
    double sr, sg, sb;
    double dr, dg, db;

    sr = (double)((src >> 0) & 3);
    sg = (double)((src >> 2) & 3);
    sb = (double)((src >> 4) & 3);
            
    dr = (double)((dst >> 0) & 3);
    dg = (double)((dst >> 2) & 3);
    db = (double)((dst >> 4) & 3);

    double rdist = (dr-sr);
    double gdist = (dg-sg);
    double bdist = (db-sb);

    double rstep = (rdist / 3.0f);
    double gstep = (gdist / 3.0f);
    double bstep = (bdist / 3.0f);

    double xr = sr + rstep * step;
    double xg = sg + gstep * step;
    double xb = sb + bstep * step;

    if(xr < 0.0f) xr = 0.0f;
    if(xr > 3.0f) xr = 3.0f;
    if(xg < 0.0f) xg = 0.0f;
    if(xg > 3.0f) xg = 3.0f;
    if(xb < 0.0f) xb = 0.0f;
    if(xb > 3.0f) xb = 3.0f;

    int or = (int)xr;
    int og = (int)xg;
    int ob = (int)xb;

    int cramval = (ob << 4 | og << 2 | or << 0);

#if DEBUG
    printf("%d S>%.0f,%.0f,%.0f D>%.0f,%.0f,%.0f | X>%.2f,%.2f,%.2f D>%.2f,%.2f,%.2f S>%.2f,%.2f,%.2f %02X\n",
        step,
        sr,sg,sb,
        dr,dg,db,
        xr, xg, xb,
        rdist,bdist,gdist,
        rstep,gstep,bstep,
        cramval
        );
#endif
    return cramval;
}

void make_ramp(void)
{
    int step;
    int index;
    int seq;

    for(seq = 0; seq < 4; seq++)
    {
        for(step = 0; step < 4; step++)
        {
            for(index = 0; index < 4; index++)
            {
                ramp[seq][step*4+0] = bg_cram[step];
                ramp[seq][step*4+1] = interpolate(bg_cram[step], fg_cram, seq);
                ramp[seq][step*4+2] = interpolate(fg_cram, bg_cram[step], seq);
                ramp[seq][step*4+3] = fg_cram;
            }
        }
    }

#if DEBUG
    int x, y, z;
    for(x = 0; x < 4; x++)
    {
        for(y = 0; y < 4; y++)
        {
            for(z = 0; z < 4; z++)
            {
                int k = ramp[x][y*4+z];
                printf("%02X, ", k);
            }
            printf("\n");
        }
        printf("\n");
    }
#endif

    FILE *fd;
    fd = fopen("ramp.bin", "wb");
    fwrite(ramp, sizeof(ramp), 1, fd);
    fclose(fd);
}

int main (int argc, char *argv[])
{
    make_ramp();
    return 0;
}
