
#include <stdio.h>
#include <allegro.h>

typedef unsigned char uint8;
typedef unsigned short int uint16;

BITMAP *fg;
BITMAP *bg;
BITMAP *alt;
BITMAP *tmp;
BITMAP *bmp;
PALETTE pal;
uint8 cram[32];
PALETTE cram_pal;
PALETTE screen_pal;
BITMAP *frames[32];
uint8 sine32[256];
uint8 vram[0x4000];


void load_tables(void)
{
    FILE *fd;
    fd = fopen("sine32.pce", "rb");
    if(!fd) exit(1);
    fread(sine32, 256, 1, fd);
    fclose(fd);
}


void make_vram(void)
{
    int i, x, y;
    int k;
    int row, col;

    k = 0;
    for(i = 0; i < 32; i++)
    {
        for(row = 0; row < 4; row++)
        {
            for(col = 0; col < 4; col++)
            {
                for(y = 0; y < 8; y++)
                {
                    uint8 b0 = 0;
                    uint8 b1 = 0;
                    uint8 b2 = 0;
                    uint8 b3 = 0;
                    for(x = 0; x < 8; x++)
                    {
                        int c = getpixel(frames[i], col*8+x, row*8+y);
                        int i0 = (c >> 0) & 1;
                        int i1 = (c >> 1) & 1;
                        int i2 = (c >> 2) & 1;
                        int i3 = (c >> 3) & 1;
                        b0 |= (i0 << (x ^ 7));
                        b1 |= (i1 << (x ^ 7));
                        b2 |= (i2 << (x ^ 7));
                        b3 |= (i3 << (x ^ 7));
                    }
                    vram[(k*32)+(y*4)+0] = b0;
                    vram[(k*32)+(y*4)+1] = b1;
                    vram[(k*32)+(y*4)+2] = b2;
                    vram[(k*32)+(y*4)+3] = b3;
                }
                k++;
            }
        }
    }

    FILE *fd;
    fd = fopen("cake.anm", "wb");
    fwrite(vram, sizeof(vram), 1, fd);
    fclose(fd);
}

void make_frames(void)
{
    int i;
    int x, y;
    int sub;

        for(i = 0; i < 32; i++)
        {   
            frames[i] = create_bitmap(32, 32);
            clear(frames[i]);

            for(y = 0; y < 32; y++)
            {
                int yscroll  = (y + i) & 0x1F;
            
                for(x = 0; x < 32; x++)
                {
                    int mask = getpixel(fg, x, yscroll);
                    int front = getpixel(fg, x, yscroll);

                    int back = getpixel(bg, (x)&0x1f, (y)&0x1f);

                    switch(back)
                    {
                        case 0x00:
                                back = 0;
                                break;
                        case 0x3f:
                                back = 1;
                                break;
                        case 0x7f:
                                back = 2;
                                break;
                        case 0xff:
                                back = 3;
                                break;
                    }

                    int alter = getpixel(alt, x, yscroll);
                    int alter_mask = getpixel(alt, x, yscroll);

                    alter = (alter == 0xff) ? 1 : 0;
                    alter_mask = (alter_mask == 0x7f) ? 1 : 0;

                    mask = (mask == 0x7F) ? 1 : 0;
                    front = (front == 0xFF) ? 1 : 0;
//                  back = (back == 0xFF) ? 1 : 0;
//                  front = mask ? back : front;
//                  back = alter_mask ? back : alter;
                    int pixel = (back << 2) | alter << 1 | front << 0;
//                  int pixel = (back << 1) | front | (sub << 2);

                    putpixel(frames[i], x, y, pixel);
                }
            }
        }
}

void palette_setup(void)
{
    int i;

    for(i = 0; i < 64; i++)
    {
        int b = (i >> 4) & 3;
        int g = (i >> 2) & 3;
        int r = (i >> 0) & 3;

        r = (r << 4 | r << 2 | r << 0);
        g = (g << 4 | g << 2 | g << 0);
        b = (b << 4 | b << 2 | b << 0);
        
        cram_pal[i].r = r;
        cram_pal[i].g = g;
        cram_pal[i].b = b;
    }

    memset(cram, 0, sizeof(cram));
}

void palette_update(void)
{
    int i;
    for(i = 0; i < 32; i++)
    {
        int index = cram[i] & 0x3F;
        screen_pal[i].r = cram_pal[index].r;
        screen_pal[i].g = cram_pal[index].g;
        screen_pal[i].b = cram_pal[index].b;
    }
    set_palette(screen_pal);
}

int xfade[4][4] = {

    {0x00, 0x00, 0x00, 0x00},
    {0x3f, 0x2a, 0x15, 0x00},
    {0x00, 0x15, 0x2a, 0x3f},
    {0x3f, 0x3f, 0x3f, 0x3f},

};

void update_crossfade(int frame)
{
    int i;
    int time = (frame >> 3) & 0x0f;
    int time_seq[] = {0, 0, 0, 1, 2, 3, 3, 3, 3, 3, 3, 2, 1, 0, 0, 0};
    time = time_seq[time];

    for(i = 0; i < 16; i++)
    {
        int sub = (i >> 2) & 3;
        cram[i] = xfade[i & 3][ time ];
        cram[i] = i;
    }        
}

int main (int argc, char *argv[])
{
    int frame_count;
    int i, x, y;
    allegro_init();

    set_color_depth(8);
    set_gfx_mode(GFX_AUTODETECT_WINDOWED, 512, 384, 0, 0);
    clear(screen);

    bmp = create_bitmap(SCREEN_W/2, SCREEN_H/2);
    tmp = create_bitmap(SCREEN_W/2, SCREEN_H/2);
    clear(bmp);
    clear(tmp);

    fg = load_bmp("pie.bmp", pal);
    bg = load_bmp("dich4.bmp", pal);
    alt = load_bmp("gift.bmp", pal);
    if(!fg || !bg || !alt) 
        exit(1);

    install_timer();
    install_keyboard();

    palette_setup();    
    make_frames();
    make_vram();

    load_tables();  

    for(i = 0; i < 16; i++)
    {
        int tab[] = {0x10, 0x3f, 0x10, 0x2f, 0x10, 0x1f, 0x10, 0x0f};
        cram[i] = tab[i & 7];
    }

    frame_count = 0;

    while(1)    
    {
        if(keyboard_needs_poll())
            poll_keyboard();
        if(key[KEY_ESC])
            break;

//      int which = sine32[(frame_count + ) & 0xFF]>>1;
        int which = 0;
        int draw_pos = 0;
        int xscroll = (frame_count & 7) * 32;

        xscroll =  ((frame_count - 0) & 7) * 32;
        which = sine32[(frame_count*4) & 0xff]>>2; 
        draw_pos = ((frame_count - 1) & 7) * 32;

        for(y = 0; y < 6; y++)
            blit(frames[which & 0x1f], tmp, 0, 0, draw_pos, y*32, 32, 32);

        for(y = 0; y < 192; y++)
        {
            for(x = 0; x < 256; x++)
            {
                int c = getpixel(tmp, (x + xscroll) & 0xff, y);
                putpixel(bmp, x, y, c);
            }
        }

        stretch_blit(bmp, screen, 0, 0, bmp->w, bmp->h, 0, 0, SCREEN_W, SCREEN_H);
        update_crossfade(frame_count);
        palette_update();
        
        vsync();            
        frame_count = (frame_count + 1) & 0xFF;
    }

    return 0;
}
END_OF_MAIN();
