;----------------------------------------------------------------------------
; vdp.inc
;----------------------------------------------------------------------------

vdp_init:
                        ; Acknowledge pending interrupts
                        in      a, ($BF)

                        ; Wait for scanline $B0
        vwaitini:       in      a, ($7E)
                        cp      $B0
                        jr      nz, vwaitini

                        ; Load VDP registers
                        ld      hl, vdp_regini
                        ld      bc, $16BF
                        otir

                        ; Clear CRAM
                        ld      hl, $C000
                        rst     10h
                        ld      bc, $20BE
        clrcrmlp:       out     (c), 0
                        djnz    clrcrmlp

                        ; Clear VRAM
                        ld      hl, $4000
                        rst     10h
                        ld      d, $10
        clrvrmhi:       ld      bc, $00BE
        clrvrmlp:       out     (c), 0
                        out     (c), 0
                        out     (c), 0
                        out     (c), 0
                        djnz    clrvrmlp
                        dec     d
                        jr      nz, clrvrmhi

                        ; Write sprite end-of-list marker
                        ld      hl, $7F00
                        rst     10h
                        ld      a, $D0
                        out     ($BE), a

                        ; Set palette
                        ld      hl, default_palette
                        call    set_palette

                        ret

;----------------------------------------------------------------------------
; Subroutines
;----------------------------------------------------------------------------

; hl = palette
set_palette:            push    hl
                        ld      hl, $C000
                        rst     10h
                        pop     hl
                        ld      bc, $20BE
                        otir
                        ret

;----------------------------------------------------------------------------
; Data
;----------------------------------------------------------------------------

vdp_regini:             .dw     $8004
                        .dw     $8100
                        .dw     $82FF
                        .dw     $83FF
                        .dw     $84FF
                        .dw     $85FF
                        .dw     $86FB
                        .dw     $870F
                        .dw     $8800
                        .dw     $8900
                        .dw     $8AFF

default_palette:        ; Background palette
                        defcol  1,1,1
                        defcol  3,3,3
                        defcol  0,0,0
                        defcol  0,0,0
                        defcol  0,0,0
                        defcol  0,0,0
                        defcol  0,0,0
                        defcol  0,0,0
                        defcol  0,0,0
                        defcol  0,0,0
                        defcol  0,0,0
                        defcol  0,0,0
                        defcol  0,0,0
                        defcol  0,0,0
                        defcol  0,0,0
                        defcol  0,0,2



                        ; Sprite palette
                        defcol  0,0,1
                        defcol  0,0,0
                        defcol  0,0,0
                        defcol  0,0,0
                        defcol  0,0,0
                        defcol  0,0,0
                        defcol  0,0,0
                        defcol  0,0,0
                        defcol  0,0,0
                        defcol  0,0,0
                        defcol  0,0,0
                        defcol  0,0,0
                        defcol  0,0,0
                        defcol  0,0,0
                        defcol  0,0,0
                        defcol  0,0,2



;----------------------------------------------------------------------------
; End
;----------------------------------------------------------------------------






